@@ -4,7 +4,7 @@ import numpy
 import StringIO
 from collections import defaultdict
 
-import hydrat.common.markup as markup
+import hydrat.display.markup as markup
 from hydrat.display.html import TableSort
 from hydrat.common import as_set
 from hydrat.display.tsr import result_summary_table, project_compound
@@ -14,12 +14,13 @@ from hydrat.common.metadata import metamap
 import hydrat.result.stats as stats
 from display import list_as_html, dict_as_html, list_of_links
 from common import page_config
+from hydrat.store import NoData
 
 
 KEY_SEP =':'
 
 def results_metadata_map(store, params, max_uniq = 10):
-  mapping = metamap( store._get_TaskSetResultMetadata(uuid) for uuid in store._resolve_TaskSetResults(params) )
+  mapping = metamap( t.metadata for t in store.get_TaskSetResults(params) )
   for key in mapping.keys():
     if len(mapping[key]) > max_uniq or len(mapping[key]) <= 1:
       del mapping[key]
@@ -29,14 +30,14 @@ from hydrat.summary import Summary
 class Navigation(Summary):
   def init(self, result, interpreter):
     Summary.init(self, result, interpreter)
-    self.uuid = str(result.metadata['uuid'])
+    self.uuid = str(result.uuid)
 
   def key__link(self):
     link = markup.oneliner.a('link', href='view?'+urllib.urlencode({'uuid':self.uuid}))
     return str(link)
     
   def key__pairs(self):
-    link = markup.oneliner.a('link', href='matrix?'+urllib.urlencode({'uuid':self.uuid}))
+    link = markup.oneliner.a('link', href='confusion?'+urllib.urlencode({'uuid':self.uuid}))
     return str(link)
 
   def key__select(self):
@@ -69,6 +70,9 @@ class Results(object):
 
     summaries = self.get_full_summary(uuids, summary_fn)
 
+    page.h3('Interpreter')
+    page.add(dict_as_html({'Interpreter':int_id}))
+
     page.h3('Parameters')
     page.add(dict_as_html(params))
 
@@ -273,6 +277,10 @@ class Results(object):
         page.add(dict_as_html({goldstandard:mcnemar_pc[gs_i]}))
       else:
         page.add(dict_as_html(dict(zip(clabels, mcnemar_pc))))
+        
+      # Added by Li Wang (li@liwang.info)
+      randomisation_result = stats.randomisation(self.interpreter, results[0], results[1])
+      page.add(dict_as_html(dict(randomisation=randomisation_result)))
     else:
       page.p("ANOVA")
 
@@ -337,7 +345,7 @@ class Results(object):
         if goldstandard is None or goldstandard == label:
           with page.tr:
             with page.th:
-              link = '../datasets/'+md[ds_key]+'/instances?'+urllib.urlencode({'id':instance_id})
+              link = '../datasets/'+md[ds_key]+'/instances/'+instance_id
               page.a(instance_id, href= link)
 
             with page.td:
@@ -382,7 +390,8 @@ class Results(object):
     summaries = []
 
     for uuid in uuids:
-      summary = self.store.get_Summary(uuid, int_id)
+      result = self.store._get_TaskSetResult(uuid)
+      summary = result.summaries[int_id]
       missing_keys = set(summary_fn.keys) - set(summary)
       if len(missing_keys) > 0:
         result = self.store._get_TaskSetResult(uuid)
@@ -431,24 +440,178 @@ class Results(object):
       
   @cherrypy.expose
   def view(self, uuid):
+    """
+    Display details for a single TSR
+    """
     from hydrat.display.tsr import render_TaskSetResult
+    
+    tsr = self.store._get_TaskSetResult(uuid)
+    confusion_matrix = tsr.overall_confusion_matrix(self.interpreter).sum(0)
+    used_classes = (confusion_matrix[:,0] + confusion_matrix[:,3]) > 0
+    summary = self.summary_fn(tsr, self.interpreter)
+    from hydrat.result import CombinedMacroAverage, CombinedMicroAverage, PRF
+    metric = PRF()
+
+    rows = []
+    footer = defaultdict(int)
+    for i,cm in enumerate(confusion_matrix[used_classes]):
+      row = {}
+      #row['label'] = class_space[i]
+      row['tp'], row['tn'], row['fp'], row['fn'] = cm
+      row['precision'], row['recall'], row['fscore'] = metric(cm)
+      footer['tp'] += row['tp']
+      footer['tn'] += row['tn']
+      footer['fp'] += row['fp']
+      footer['fn'] += row['fn']
+      rows.append(row)
+
+    cols, col_headings = zip(*[
+      ('tp',dict(label='TP', sorter='digit')),
+      ('tn',dict(label='TN', sorter='digit')),
+      ('fp',dict(label='FP', sorter='digit')),
+      ('fn',dict(label='FN', sorter='digit')),
+      ('precision', dict(label='P', sorter='digit')),
+      ('recall', dict(label='R', sorter='digit')),
+      ('fscore', dict(label='F', sorter='digit')),
+      ])
+
     page = markup.page()
     page.init(**page_config)
-    
+
+    try:
+      class_space = self.store.get_Space(tsr.metadata['class_space'])
+    except NoData:
+      class_space = ['class{0}'.format(i) for i in xrange(confusion_matrix.shape[0])]
+      page.add('<h1>Warning: No data for class space "{0}"</h1>'.format(tsr.metadata['class_space']))
+
+    # Data summmary
+    page.add(dict_as_html(summary))
+
+    # Overall summary
+    space = list(numpy.array(class_space)[used_classes])
+    page.dict_table(rows, cols, list(col_headings), space, footer)
+
+    # Multiclass summary
+    # TODO: If either the goldstandard or the output are multiclass,
+    #       we will display a breakdown of the confusion matrix
+    #       according to (x,y) -> (a,b,c).
+
+
+    for i, result in enumerate(tsr.results):
+      page.h2('Result %d' % i)
+      cm_r = result.confusion_matrix(self.interpreter)
+      used_classes = (cm_r[:,0] + cm_r[:,3]) > 0
+      space_r = list(numpy.array(class_space)[used_classes])
+      rows_r = []
+      footer_r = defaultdict(int)
+      for i, cm in enumerate(cm_r[used_classes]):
+        row = {}
+        row['tp'], row['tn'], row['fp'], row['fn'] = cm
+        row['precision'], row['recall'], row['fscore'] = metric(cm)
+        footer_r['tp'] += row['tp']
+        footer_r['tn'] += row['tn']
+        footer_r['fp'] += row['fp']
+        footer_r['fn'] += row['fn']
+        rows_r.append(row)
+      footer_cm = footer_r['tp'],footer_r['tn'], footer_r['fp'], footer_r['fn']
+      footer_r['precision'], footer_r['recall'], footer_r['fscore'] = metric(numpy.array(footer_cm))
+
+      with page.table:
+        with page.tr:
+          with page.td:
+            #TODO: More info here
+            page.add(dict_as_html(result.metadata))
+          with page.td:
+            page.dict_table(rows_r, cols, list(col_headings), space_r, footer_r)
+      
+
+
+
+    #text = StringIO.StringIO()
+    #with TableSort(text) as result_renderer:
+    #  render_TaskSetResult(result_renderer, result, class_space, self.interpreter, summary)
+
+    #page.add(text.getvalue())
+    return str(page)
+
+  @cherrypy.expose
+  def confusion(self, uuid):
+    """
+    Display a sortable list of confusion pairs.
+    """
     result = self.store._get_TaskSetResult(uuid)
+    # TODO: make the class space a result attribute
     class_space = self.store.get_Space(result.metadata['class_space'])
     summary = self.summary_fn(result, self.interpreter)
-    text = StringIO.StringIO()
-    with TableSort(text) as result_renderer:
-      render_TaskSetResult(result_renderer, result, class_space, self.interpreter, summary)
+    pairs = result.overall_classpairs(self.interpreter)
 
-    page.add(text.getvalue())
-    return str(page)
+    rows = []
+    fr_count = defaultdict(int)
+    to_count = defaultdict(int)
+    fp_count = defaultdict(int)
+    fn_count = defaultdict(int)
+    for cl_fr, cl_to in pairs:
+      count = len(pairs[cl_fr, cl_to])
+      fr_count[class_space[cl_fr]] += count
+      to_count[class_space[cl_to]] += count
+      if cl_fr != cl_to:
+        fp_count[class_space[cl_to]] += count
+        fn_count[class_space[cl_fr]] += count
+        row = {}
+        #TODO: Construct useful links here!!
+        row['from'] = markup.oneliner.a(class_space[cl_fr], href='')
+        row['to'] = markup.oneliner.a(class_space[cl_to], href='')
+        link = 'classpair?'+urllib.urlencode({'uuid':result.uuid, 'gs':class_space[cl_fr], 'cl':class_space[cl_to]})
+        row['count'] = markup.oneliner.a(count, href=link)
+        rows.append(row)
+
+    classes = sorted(map(class_space.__getitem__, set.union(*map(set,zip(*pairs)))))
+
+    page = markup.page()
+    page.init(**page_config)
+    page.add(dict_as_html(summary))
+
+    def highlight_zero(content):
+      if content == 0:
+        page.td(content, **{'class':'highlight'})
+      else:
+        page.td(content)
+
+    with page.table:
+      with page.tr:
+        page.th('Class')
+        for c in classes:
+          link = 'classpair?'+urllib.urlencode({'uuid':result.uuid, 'gs':c, 'cl':c})
+          page.th(markup.oneliner.a(c, href=link))
+      with page.tr:
+        page.th('Goldstandard')
+        for c in classes:
+          highlight_zero(fr_count[c])
+      with page.tr:
+        page.th('Classified')
+        for c in classes:
+          highlight_zero(to_count[c])
+      with page.tr:
+        page.th('False Positive')
+        for c in classes:
+          highlight_zero(fp_count[c])
+      with page.tr:
+        page.th('False Negative')
+        for c in classes:
+          highlight_zero(fn_count[c])
+    page.dict_table(rows, ['from','to','count'], 
+        col_headings=[
+          {'label':'From', 'searchable':True}, 
+          {'label':'To', 'searchable':True}, 
+          'Count',
+          ])
+    return page()
 
   @cherrypy.expose
   def matrix(self, uuid, threshold=0):
     threshold = int(threshold)
     result = self.store._get_TaskSetResult(uuid)
+    summary = self.summary_fn(result, self.interpreter)
     class_space = self.store.get_Space(result.metadata['class_space'])
     matrix = result.overall_classification_matrix(self.interpreter)
     matrix_sans_diag = numpy.logical_not(numpy.diag(numpy.ones(len(class_space), dtype=bool))) * matrix
@@ -459,6 +622,7 @@ class Results(object):
 
     page = markup.page()
     page.init(**page_config)
+    page.add(dict_as_html(summary))
     with page.table:
       with page.tr:
         page.th()
@@ -488,7 +652,7 @@ class Results(object):
     if 'eval_dataset' in result.metadata:
       dataset = result.metadata['eval_dataset']
     else:
-      dataset = result.metadata['dataset']
+      dataset = result.metadata['instance_space']
     docids = list(self.store.get_InstanceIds(dataset))
     pairs = result.overall_classpairs(self.interpreter)
 
@@ -507,7 +671,7 @@ class Results(object):
         with page.tr:
           id = docids[i]
           with page.th:
-            page.a(id, href='../datasets/%s/instances?' % dataset + urllib.urlencode({'id':id}))
+            page.a(id, href='../datasets/%s/instances/%s' % (dataset, id))
           for ts in tokenstreams:
             with page.td:
               page.a(ts,href='../datasets/%s/tokenstream/%s/%s' % (dataset, ts, id))
