@@ -3,6 +3,7 @@ import logging
 import warnings
 import functools
 import inspect
+import time
 from hydrat import config
 
 
@@ -111,6 +112,52 @@ class replace_with_result(object):
         return self.fn.__doc__ + " (cached)"
 
 
+class timed(object):
+  """ Decorator that adds execution time of a method to a
+      dictionary called __timing_data__
+  """
+  def __init__(self, decoratee):
+    self.decoratee = decoratee
+    self.__name__ = decoratee.__name__
+    self.instance = None
+    self.owner = None
+
+  def __get__(self, instance, owner):
+    self.instance = instance
+    self.owner = owner
+    return self
+
+  def __call__(self, *args, **kwargs):
+    if self.owner is None:
+      # If we have no owner, we are decorating a function
+      # TODO: Can we inject into a global in this case?
+      raise NotImplementedError, "Only implemented for methods"
+
+    if self.instance:
+      # If we have an instance we are bound
+      instance = self.instance
+    else:
+      # Otherwise, we are unbound and the first argument is the instance
+      instance = args[0]
+      args = args[1:]
+
+    # Do the actual operation
+    start = time.time()
+    retval = self.decoratee(instance, *args, **kwargs)
+    duration = time.time() - start
+
+    # Create the __timing_data__ dictionary if needed
+    if not hasattr(instance, '__timing_data__'):
+      instance.__timing_data__ = {}
+
+    # Attach the measured time
+    instance.__timing_data__[self.__name__] = duration
+
+    return retval
+
+
+
+
 # from http://wiki.python.org/moin/PythonDecoratorLibrary#GeneratingDeprecationWarnings
 def deprecated(func):
     """This is a decorator which can be used to mark functions
@@ -133,3 +180,17 @@ def deprecated(func):
 
         return func(*args, **kwargs)
     return new_func
+
+# from http://wiki.python.org/moin/PythonDecoratorLibrary#Synchronization
+def synchronized(lock):
+    """ Synchronization decorator. """
+
+    def wrap(f):
+        def new_function(*args, **kw):
+            lock.acquire()
+            try:
+                return f(*args, **kw)
+            finally:
+                lock.release()
+        return new_function
+    return wrap
