@@ -3,6 +3,7 @@ import logging
 import os
 import sys
 import numpy
+import multiprocessing as mp
 
 logger = logging.getLogger(__name__)
 
@@ -22,6 +23,8 @@ class HydratConfigError(Exception): pass
 # (http://www.richardmurri.com/projects/find/), or maybe DIY. Would be nice
 # to specify things like 'md5sum' in addition to the filename. Could also
 # look into gentoo portage for inspiration.
+#
+# TODO: Make .hydratrc behaviour cascade, so that local .hydratrc supercedes parent supercedes global
 
 
 class DIR(object):
@@ -161,10 +164,17 @@ def default_configuration():
   """
   default_config = HydratConfigParser()
 
+  default_config.add_section('parameters')
+  default_config.set('parameters', 'job_count', str(mp.cpu_count()))
+  default_config.set('parameters', 'parallel_classify', 'True')
+  default_config.set('parameters', 'parallel_tokenize', 'True')
+  default_config.set('parameters', 'max_feats', '50000')
+
   default_config.add_section('paths')
   default_config.set('paths', 'scratch', '/tmp')
   default_config.set('paths', 'output', './output')
   default_config.set('paths', 'corpora', '~/data')
+  default_config.set('paths', 'store', '.')
 
   default_config.add_section('tools')
   default_config.set('tools', 'rainbow', '/usr/bin/rainbow')
@@ -197,7 +207,10 @@ def default_configuration():
   default_config.set('debug', 'pdb_on_unhandled_exception', 'False')
   default_config.set('debug', 'clear_temp_files', 'True')
   default_config.set('debug', 'allow_str_classset', 'False')
+  default_config.set('debug', 'allow_empty_instance', 'False')
+  default_config.set('debug', 'allow_decode_error', 'False')
   default_config.set('debug', 'pdb_on_summaryfn_exception', 'False')
+  default_config.set('debug', 'quiet_extern', 'True')
 
   return default_config
 
@@ -215,7 +228,8 @@ def read_configuration(additional_path=[]):
   """
   if isinstance(additional_path, str):
     additional_path = [additional_path]
-  all_paths = [os.path.expanduser('~/'+DEFAULT_CONFIG_FILE),'./'+DEFAULT_CONFIG_FILE ]
+  all_paths = [os.path.expanduser('~/'+DEFAULT_CONFIG_FILE),
+      './'+DEFAULT_CONFIG_FILE,'../'+DEFAULT_CONFIG_FILE, ]
   all_paths.extend(additional_path)
 
   config = default_configuration()
@@ -289,7 +303,13 @@ def load_configuration(config):
   console_output.setFormatter(console_formatter)
 
   if config.get('logging','logfile'):
-    logfile_output = logging.FileHandler(config.getpath('logging','logfile'), delay=True)
+    # pull out the name of the calling file
+    # adapted from store.py
+    import inspect
+    stack = inspect.stack()
+    filename = os.path.splitext(os.path.basename(stack[-1][1]))[0] + '.' + config.getpath('logging','logfile')
+
+    logfile_output = logging.FileHandler(filename, delay=True)
     logfile_level = LEVELS.get(config.get('logging','logfile.level'), logging.NOTSET)
     logfile_formatter = logging.Formatter(config.get('logging','logfile.format',raw=True))
     logfile_output.setLevel(logfile_level)
@@ -329,4 +349,13 @@ def load_configuration(config):
     # TODO: Test writable?
     raise HydratConfigError, "Scratch path '%s' does not exist" % scratchpath
    
-
+# Import of the browser configuration
+# TODO: This really should not be here
+#import sys
+#sys.path.append('.')
+#try:
+#  import browser_config
+#except ImportError:
+#  logger.debug('did not find a local browser_config.py')
+#  import hydrat.browser.browser_config as browser_config
+#del sys.path[-1]
